<?php
/**
 * montonio/module-hire-purchase
 *
 * @category   montonio
 * @package    module-hire-purchase
 * @copyright  Copyright (c) 2020 Montonio Finance OÜ  (https://montonio.com/)
 * @license    https://www.gnu.org/licenses/gpl-3.0.txt
 */
namespace Montonio\Hirepurchase\Block\Adminhtml\System\Config\Form\Field;

/**
 * @inheritdoc
 */
class AvailableMethods extends \Magento\Config\Block\System\Config\Form\Field\FieldArray\AbstractFieldArray
{

    /**
     *
     * @param \Montonio\Hirepurchase\Gateway\Payments\Command\AspspsCommand $aspspsCommand
     * @param \Magento\Framework\View\Element\BlockFactory $blockFactory
     * @param \Magento\Framework\Data\Form\Element\Factory $elementFactory
     * @param \Magento\Framework\View\Design\Theme\LabelFactory $labelFactory
     * @param \Magento\Backend\Block\Template\Context $context
     * @param array $data
     */
    public function __construct(
        \Montonio\Hirepurchase\Gateway\Payments\Command\AspspsCommand $aspspsCommand,
        \Magento\Framework\View\Element\BlockFactory $blockFactory,
        \Magento\Framework\Data\Form\Element\Factory $elementFactory,
        \Magento\Framework\View\Design\Theme\LabelFactory $labelFactory,
        \Magento\Backend\Block\Template\Context $context,
        array $data = []
    ) {

        $this->aspspsCommand = $aspspsCommand;
        $this->blockFactory = $blockFactory;
        $this->elementFactory = $elementFactory;
        $this->labelFactory = $labelFactory;

        $this->setTemplate('Montonio_Hirepurchase::system/config/form/field/array.phtml');

        parent::__construct($context, $data);
    }

    public function getArrayRows()
    {

        $values = $this->getElement()->getValue() ?? [];

        try {
            $methods = $this->aspspsCommand->execute([]);
            foreach ($methods as $k => $v) {
                if (!isset($values[$v['bic']])) {
                    $values[$v['bic']] = [
                        'method_enabled'=> '1',
                        'method_title'=> $v['name'],
                        'method_desc' => $v['name'],
                        'logo_url'    => $v['logo_url']
                    ];
                }
            }
        } catch (\Exception $e) {
            //silence
            $this->silence();
        }

        $this->getElement()->setValue($values);

        return parent::getArrayRows();
    }

    /**
     * Prepare to render
     *
     * @return void
     */
    public function _prepareToRender(): void
    {
        $elementBlockClass = \Magento\Framework\Data\Form\Element\Checkbox::class;

        $this->addColumn('method_enabled', [
            'label' => __('On/Off'),
            'style' => 'width: 15px'
        ]);

        $this->addColumn('method_title', [
            'label' => __('Title'),
            'style' => 'width: 150px'
        ]);

        $this->addColumn('method_desc', [
            'label' => __('Description'),
            'style' => 'width: 150px'
        ]);

        $this->addColumn('logo_url', [
            'label' => __('Logo Url'),
            'style' => 'width: 150px'
        ]);

        $this->_addAfter = false;
        $this->_addButtonLabel = __('Add Fields');
    }

    /**
     * Render array cell for prototypeJS template
     *
     * @param string $columnName
     * @return string
     */
    public function renderCellTemplate($columnName)
    {
        if ($columnName == 'method_enabled' && isset($this->_columns[$columnName])) {
            /** @var $label \Magento\Framework\View\Design\Theme\Label */
            $label = $this->labelFactory->create();
            $options = ['0' => __('Off'), '1' => __('On')];
            $element = $this->elementFactory->create('select');
            $element->setForm($this->getForm())
                ->setName($this->_getCellInputElementName($columnName))
                ->setHtmlId($this->_getCellInputElementId('<%- _id %>', $columnName))
                ->setValues($options);
            return str_replace("\n", '', $element->getElementHtml());
        }

        return parent::renderCellTemplate($columnName);
    }

    /**
     * Do nothing
     * @return void
     */
    public function silence()
    {
    }
}
