<?php
/**
 * montonio/module-hire-purchase
 *
 * @category   montonio
 * @package    module-hire-purchase
 * @copyright  Copyright (c) 2020 Montonio Finance OÜ  (https://montonio.com/)
 * @license    https://www.gnu.org/licenses/gpl-3.0.txt
 */
namespace Montonio\Hirepurchase\Block\Frontend\LayoutProcessors;

/**
 * {@inheritdoc}
 */
class PaymentsProcessor implements \Magento\Checkout\Block\Checkout\LayoutProcessorInterface
{
    /**
     *
     * @param \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
     * @param \Montonio\Hirepurchase\Gateway\Payments\Config\Config $config
     */
    public function __construct(
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        \Montonio\Hirepurchase\Gateway\Payments\Config\Config $config
    ) {
            $this->scopeConfig = $scopeConfig;
            $this->config = $config;
    }

    /**
     * {@inheritdoc}
     */
    public function process($jsLayout)
    {

        $enabled = $this->config->isActive();
        if (!$enabled) {
            return $jsLayout;
        }

        $methodCode = $this->config->getCode();

        $layoutPath = $jsLayout['components']['checkout']
            ['children']['steps']
            ['children']['billing-step']
            ['children']['payment']
            ['children']['renders']
            ['children'] ?? false;

        if ($layoutPath) {
            $paymentMethods = [];

            $availableMethods = $this->config->getAvailableMethods();

            //if grouped we ignore other methods
            $isGrouped = $this->config->getValue('grouped');
            if ($isGrouped) {
                $availableMethods = ['grouped' => 0];
            }

            foreach ($availableMethods as $code => $data) {
                $subMethodCode = $methodCode. '_' .$code;
                $title = (isset($data['method_title'])) ? $data['method_title'] : false;
                $description = (isset($data['method_desc'])) ?
                    $data['method_desc'] : $this->config->getServiceDescription();
                $logoPath = (isset($data['logo_url'])) ? $data['logo_url'] : $this->config->getServiceLogoPath();
                $paymentMethods[$subMethodCode] = [
                    'component' => 'Montonio_Hirepurchase/js/view/payment/montonio',
                    'cnf' => [
                        'methodCode' => $subMethodCode,
                        'title' => $title,
                        'serviceDescription' => $description,
                        'serviceLogoPath' => $logoPath
                    ]
                ];
            }
            $paymentMethods = array_merge($layoutPath, $paymentMethods);

            $jsLayout['components']['checkout']
                ['children']['steps']
                ['children']['billing-step']
                ['children']['payment']
                ['children']['renders']
                ['children'] = $paymentMethods;
        }
        return $jsLayout;
    }
}
