<?php
/**
 * montonio/module-hire-purchase
 *
 * @category   montonio
 * @package    module-hire-purchase
 * @copyright  Copyright (c) 2020 Montonio Finance OÜ  (https://montonio.com/)
 * @license    https://www.gnu.org/licenses/gpl-3.0.txt
 */
declare(strict_types=1);

namespace Montonio\Hirepurchase\Plugin\Order;

/**
 * Handle getCanSendNewEmailFlag false for pending orders
 */
class ControlNewEmailPlugin
{
    /**
     *
     * @param \Montonio\Hirepurchase\Model\Payment\ConfigProvider $configProvider
     * @param \Montonio\Hirepurchase\Model\Payment\PaymentsConfigProvider $paymentsConfigProvider
     * @param \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
     */
    public function __construct(
        \Montonio\Hirepurchase\Model\Payment\ConfigProvider $configProvider,
        \Montonio\Hirepurchase\Model\Payment\PaymentsConfigProvider $paymentsConfigProvider,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
    ) {
        $this->configProvider = $configProvider;
        $this->paymentsConfigProvider = $paymentsConfigProvider;
        $this->scopeConfig = $scopeConfig;
    }

    /**
     *
     * @param \Magento\Sales\Model\Order $subject
     * @param callable $proceed
     * @return boolean
     */
    public function aroundGetCanSendNewEmailFlag(
        \Magento\Sales\Model\Order $subject,
        callable $proceed
    ) {

        //default on all other cases
        $result = $proceed();

        $targetMethod = $subject
            ->getPayment()
            ->getMethodInstance()
            ->getCode() ?? false;

        $orderState = \Magento\Sales\Model\Order::STATE_NEW;
        $orderState = ($subject->getState() == $orderState) ? true : false;

        //if order status new
        if ($orderState) {

            //payments
            $methodCode = $this->paymentsConfigProvider::CODE;
            //this code is virtual, match only if inside a string
            $targetMethod = strpos($targetMethod, $methodCode) !== false
                                ? $methodCode : $targetMethod;
            $enabled = $this->paymentsConfigProvider->getConfigObject()->isActive();
            $allowed = $this->paymentsConfigProvider
                        ->getConfigObject()
                        ->getValue('order_email_after');

            //if payment method is enabled
            //if method is right
            //if e-mail is allowed
            //set it to false
            if ($enabled &&
                $targetMethod &&
                $targetMethod ==  $methodCode &&
                $allowed
            ) {
                $result = false;
            }

            //hirepurchase
            $methodCode = $this->configProvider::CODE;
            $enabled = $this->configProvider->getConfigObject()->isActive();
            $allowed = $this->configProvider
                        ->getConfigObject()
                        ->getValue('order_email_after');

            //if payment method is enabled
            //if method is right
            //if e-mail is allowed
            //set it to false
            if ($enabled &&
                $targetMethod &&
                $targetMethod == $methodCode &&
                $allowed
            ) {
                $result = false;
            }
        }

        return $result;
    }
}
