<?php
/**
 * montonio/module-hire-purchase
 *
 * @category   montonio
 * @package    module-hire-purchase
 * @copyright  Copyright (c) 2020 Montonio Finance OÜ  (https://montonio.com/)
 * @license    https://www.gnu.org/licenses/gpl-3.0.txt
 */
declare(strict_types=1);
namespace Montonio\Hirepurchase\Plugin\Virtual;

/**
 * Handle plugins on getting virtual instances on payment methods list
 */
class PaymentHelper
{
    /**
     *
     * @param \Magento\Framework\Controller\Result\RedirectFactory $resultRedirectFactory
     * @param \Magento\Framework\UrlInterface $urlInterface
     * @param \Magento\Payment\Model\Method\Factory $paymentMethodFactory
     * @param \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
     * @param \Montonio\Hirepurchase\Gateway\Payments\Config\Config $config
     */
    public function __construct(
        \Magento\Framework\Controller\Result\RedirectFactory $resultRedirectFactory,
        \Magento\Framework\UrlInterface $urlInterface,
        \Magento\Payment\Model\Method\Factory $paymentMethodFactory,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        \Montonio\Hirepurchase\Gateway\Payments\Config\Config $config
    ) {
        $this->resultRedirectFactory = $resultRedirectFactory;
        $this->urlInterface = $urlInterface;
        $this->methodFactory = $paymentMethodFactory;
        $this->scopeConfig = $scopeConfig;
        $this->config = $config;
    }

    /**
     *
     * @param \Magento\Payment\Helper\Data $subject
     * @param callable $proceed
     * @return callable | resultRedirect
     */
    public function aroundGetPaymentMethods(\Magento\Payment\Helper\Data $subject, callable $proceed)
    {

        $result = $proceed();

        $enabled = $this->config->isActive();

        $methodCode = $this->config->getCode();
        //add our virtual codes to config list based on our actual shared class

        if ($enabled && !empty($result[$methodCode])) {

            $availableMethods = $this->config->getAvailableMethods();
            $isGrouped = $this->config->getValue('grouped');

            if ($isGrouped) {
                $availableMethods = ['grouped' => '0'];
            }

            foreach ($availableMethods as $code => $data) {
                $subMethodCode = $methodCode. '_' .$code;
                $result[$subMethodCode]=$result[$methodCode];
            }

        }

        return $result;
    }

    /**
     *
     * @param \Magento\Payment\Helper\Data $subject
     * @param callable $proceed
     * @param string $code
     * @return unknown
     */
    public function aroundGetMethodInstance(\Magento\Payment\Helper\Data $subject, callable $proceed, $code)
    {

        //if the code is within our virtual codes
        //map the code to our general class and set the code
        if (strstr($code, 'montonio_payments_')) {
            $realCode = 'montonio_payments';
            $class = $this->scopeConfig->getValue(
                $this->getMethodModelConfigName($realCode, $subject),
                \Magento\Store\Model\ScopeInterface::SCOPE_STORE
            );
            $method = $this->methodFactory->create($class, ['code' => $code]);
        } else {
            //go for defaults
            $method = $proceed($code);
        }

        return $method;
    }

    /**
     * Get config name of method model
     *
     * @param string $code
     * @param \Magento\Payment\Helper\Data $scope
     * @return string
     */
    protected function getMethodModelConfigName($code, $scope)
    {
        //we just needed this to make our life easier
        return sprintf('%s/%s/model', $scope::XML_PATH_PAYMENT_METHODS, $code);
    }
}
